//
// GameX - Buffer Class Code 
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source License. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 

#include "gamex-buffer.h"
#include "gamex-global-def.h"



Buffer::Buffer ()
{
	buf_type = BUFFER_EMPTY;
	SetOrder (BUFFER_ORDER_LBF);
}




void Buffer::SetOrder (int order)
{
	byte_order = order;
	if(byte_order == BUFFER_ORDER_LBF) {		
		FuncReadC = &ReadC_LBF;		FuncWriteC = &WriteC_LBF;
		FuncReadI = &ReadI_LBF;		FuncWriteI = &WriteI_LBF;
		FuncReadL = &ReadL_LBF;		FuncWriteL = &WriteL_LBF;
	}
	else
	{
		FuncReadC = &ReadC_MBF;		FuncWriteC = &WriteC_MBF;
		FuncReadI = &ReadI_MBF;		FuncWriteI = &WriteI_MBF;
		FuncReadL = &ReadL_MBF;		FuncWriteL = &WriteL_MBF;
	}
}


void Buffer::Invert (unsigned char *in, int width)
{
	unsigned char *in_pnt;
	for(in_pnt = in; in_pnt < in+width; in_pnt++)
		*in_pnt = 255 - (*in_pnt);
}

void Buffer::Scale (unsigned char *in, int width, int scale)
{
	unsigned char *in_pnt;
	for(in_pnt = in; in_pnt < in+width; in_pnt++)
		*in_pnt *= scale;
}


int Buffer::UnstuffEachBit (unsigned char *in, unsigned char *out, int width)
{
	unsigned char *in_pnt = in;
	unsigned char *out_pnt = out;
	unsigned char bit;
	int n, j;

	for(n = 0; n < width; n++)
	{
		for(j = 0; j < 8; j++)
		{
			bit = ((*in_pnt) & Mask[j]) >> (7-j);
			*out_pnt++ = bit;
		}
		*in_pnt++;
	}
	return (width*8);
}
/*
int Buffer::UnstuffBits (unsigned char *in, unsigned char *out, int width, unsigned char *stuff_info)
{
	unsigned char *in_pnt = in;
	unsigned char *out_pnt = out;
	int n;
	
	for(n = 0; n < width; n++)
	{
		in_pnt++;
	}
	return width;
}
*/
int Buffer::ReadWord (char *line, char *word)
{
	return ReadWord (line, word, ' ');
}

int Buffer::ReadWord (char *line, char *word, char delim)
{
	char *buf_pos;
	char *start_pos;	

	// read past spaces/tabs, or until end of line/string
	for(buf_pos = line; (*buf_pos == ' ' || *buf_pos == '\t') && *buf_pos != '\n' && *buf_pos != '\0';)
		buf_pos++;

	// if end of line/string found, then no words found, return null
	if(*buf_pos == '\n' || *buf_pos == '\0') {*word = '\0'; return FALSE;}

	// mark beginning of word, read until end of word
	for(start_pos = buf_pos; *buf_pos != delim && *buf_pos != '\t' && *buf_pos != '\n' && *buf_pos != '\0';)
		buf_pos++;
	
	if(*buf_pos == '\n' || *buf_pos == '\0') {	// buf_pos now points to the end of buffer
		strcpy (word, start_pos);			// copy word to output string
		*line = '\0';						// clear input buffer
	}
	else
	{										// buf_pos now points to the delimiter after word
		*buf_pos = '\0';					// replace delimiter with end-of-word marker
		strcpy (word, start_pos);			// copy word(s) string to output string	
		buf_pos++;							// remove delimiter from buffer
											// move start_pos to beginning of entire buffer
		for(start_pos = line; *buf_pos != '\n' && *buf_pos != '\0'; )
			*start_pos++ = *buf_pos++;		// copy remainder of buffer to beginning of buffer
		*start_pos = '\0';					// mark end of new input buffer
	}
	return TRUE;						// return word(s) copied	
}
