/* Text Printer class  *\
\* By Justin Pease     */


#ifndef GAMEX_TEXT_H_INCLUDED
#define GAMEX_TEXT_H_INCLUDED

#include "gamex-global-def.h"

//#include "gamex-win-dx.h"

class TextPrinter
{
public:
	// Initialize text printer (by loading text graphics)
	// if you give charListInImageOrder, it should be a (null-terminated) string of characters in the top-to-bottom, left-to-right order they're in the image, otherwise a default order is assumed
	void Initialize(ImageX * alphabetImage, int charsAcrossImage = 16);
	void Initialize(ImageX * alphabetImage, int charsAcrossImage, const char * charListInImage);

	// Prints out a string of text graphically using the text alphabet image text_img
	// returns how many non-white-space characters were drawn
	// - fontSize is the point size of the font (dimensions in pixels of each character - may need to set this larger than you'd think necessary if your text image has much spacing around the letters)
	// - drawMode is a GameX draw mode such as DRAW_GHOST or DRAW_ADD or DRAW_SUBTRACT|DRAWOP_NOFILTER etc.
	// - if rotateBaseline is false, angle will rotate all the individual letters instead of the whole string
	// - if fixedWidth is true, the text is printed like a fixed-width font (but maybe too wide, should fix)
	int PrintText(const char * str, float x, float y, float fontSize = 24, bool centered = false, int drawMode = DRAW_ADD, ColorX & color = ColorX(), float angle = 0.0f, float yScale = 1.0f, bool rotateBaseline = true, bool fixedWidth = false);
	int PrintText3D(const char * str, const Vec3f & pos, const Vec3f & right, const Vec3f & down, float scale = 1.0f, float rOffset = 0.0f, bool centered = false, int drawMode = DRAW_ADD, ColorX & color = ColorX(), float yScale = 1.0f, bool fixedWidth = false);
	float GetVisibleCharWidth(const char * str, int index, float font_size);
	float GetVisibleStringWidth(const char * str, int first_index, int last_index, float font_size);
	float GetVisibleStringWidth(const char * str, float font_size);
	void GetVisibleCharInfo(const char * str, int n, int & xs, int & ys, float & kern, bool & visible);

	void PrintTextScreenProportional(const char * str, float x, float y, float heightRatio, float xStretch = 1.0f, bool centered = false, int draw_mode = DRAW_ADD, ColorX & color = ColorX(), float angle = 0.0f, bool rotate_baseline = true, bool fixed = false);
	float GetVisibleStringWidthProportional(const char * str, float heightRatio, float xStretch = 1.0f);
	float GetVisibleStringWidthProportional(const char * str, int first_index, int last_index, float heightRatio, float xStretch = 1.0f);

	inline void SetHighlight(ColorX c, char startChar, char endChar) {z_highlightColor = c; z_highlightStartChar = startChar; z_highlightEndChar = endChar;}

	inline bool IsInitialized() {return (text_img != NULL);}

	TextPrinter ();
	~TextPrinter ();

	ImageX * text_img;
private:

	void z_InitWithDefaultImage ();
	bool z_ownsImage;

	int z_charPixels;
	float z_fCharPixels;

	const static int z_HEIGHT_LEVELS = 32;
	struct charInfo
	{
		short xs, ys; // top-left coordinate where the character can be found in the text image
		unsigned char xl [z_HEIGHT_LEVELS], xr [z_HEIGHT_LEVELS]; // left and right max extents at given height interval of each character, normalized from 0 to 256
		bool visible;
//		inline void operator = (const charInfo& op) {xs = op.xs; ys = op.ys; memcpy(xl, op.xl, z_HEIGHT_LEVELS); memcpy(xr, op.xr, z_HEIGHT_LEVELS); visible = op.visible;}
	};
	charInfo z_chars [256];
	unsigned char z_charKern [256][256];

	ColorX z_highlightColor;
	char z_highlightStartChar, z_highlightEndChar;
	bool z_inHighlight;
	float z__avgTop, z__avgBot;
};

#endif
